@@ -20,28 +20,30 @@ import os
 import re
 import tempfile
 
-from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
 import six
 
 from os_brick import exception
+from os_brick import executor
 from os_brick.i18n import _, _LI
 
 LOG = logging.getLogger(__name__)
 
 
-class RemoteFsClient(object):
+class RemoteFsClient(executor.Executor):
 
     def __init__(self, mount_type, root_helper,
-                 execute=putils.execute, *args, **kwargs):
+                 execute=None, *args, **kwargs):
+        super(RemoteFsClient, self).__init__(root_helper, execute=execute,
+                                             *args, **kwargs)
 
         mount_type_to_option_prefix = {
             'nfs': 'nfs',
             'cifs': 'smbfs',
             'glusterfs': 'glusterfs',
             'vzstorage': 'vzstorage',
-            'scality': 'scality_sofs',
-            'quobyte': 'quobyte'
+            'quobyte': 'quobyte',
+            'scality': 'scality'
         }
 
         if mount_type not in mount_type_to_option_prefix:
@@ -60,18 +62,13 @@ class RemoteFsClient(object):
         if mount_type == "nfs":
             self._check_nfs_options()
 
-        self.root_helper = root_helper
-        self.set_execute(execute)
-
-    def set_execute(self, execute):
-        self._execute = execute
-
     def get_mount_base(self):
         return self._mount_base
 
     def _get_hash_str(self, base_str):
         """Return a string that represents hash of base_str
         (in a hex format).
+
         """
         if isinstance(base_str, six.text_type):
             base_str = base_str.encode('utf-8')
@@ -124,7 +121,7 @@ class RemoteFsClient(object):
             mnt_cmd.extend(flags)
         mnt_cmd.extend([share, mount_path])
 
-        self._execute(*mnt_cmd, root_helper=self.root_helper,
+        self._execute(*mnt_cmd, root_helper=self._root_helper,
                       run_as_root=True, check_exit_code=0)
 
     def _mount_nfs(self, nfs_share, mount_path, flags=None):
@@ -160,12 +157,12 @@ class RemoteFsClient(object):
         if os.path.exists(conf_dir):
             bs_path = os.path.join(conf_dir, 'bs_list')
             self._execute('cp', '-f', tmp_bs_path, bs_path,
-                          root_helper=self.root_helper, run_as_root=True)
+                          root_helper=self._root_helper, run_as_root=True)
         else:
             self._execute('cp', '-rf', tmp_dir, conf_dir,
-                          root_helper=self.root_helper, run_as_root=True)
+                          root_helper=self._root_helper, run_as_root=True)
         self._execute('chown', '-R', 'root:root', conf_dir,
-                      root_helper=self.root_helper, run_as_root=True)
+                      root_helper=self._root_helper, run_as_root=True)
 
     def _mount_vzstorage(self, vz_share, mount_path, flags=None):
         m = re.search("(?:(\S+):\/)?([a-zA-Z0-9_-]+)(?::(\S+))?", vz_share)
@@ -186,14 +183,14 @@ class RemoteFsClient(object):
         if passwd:
             self._execute('pstorage', '-c', cluster_name, 'auth-node', '-P',
                           process_input=passwd,
-                          root_helper=self.root_helper, run_as_root=True)
+                          root_helper=self._root_helper, run_as_root=True)
 
         mnt_cmd = ['pstorage-mount', '-c', cluster_name]
         if flags:
             mnt_cmd.extend(flags)
         mnt_cmd.extend([mount_path])
 
-        self._execute(*mnt_cmd, root_helper=self.root_helper,
+        self._execute(*mnt_cmd, root_helper=self._root_helper,
                       run_as_root=True, check_exit_code=0)
 
     def _check_nfs_options(self):
@@ -230,3 +227,37 @@ class RemoteFsClient(object):
         opt = '%s=%s' % (option, value) if value else option
         opts.append(opt)
         return ",".join(opts) if len(opts) > 1 else opts[0]
+
+
+class ScalityRemoteFsClient(RemoteFsClient):
+    def __init__(self, mount_type, root_helper,
+                 execute=None, *args, **kwargs):
+        super(ScalityRemoteFsClient, self).__init__(mount_type, root_helper,
+                                                    execute=execute,
+                                                    *args, **kwargs)
+        self._mount_type = mount_type
+        self._mount_base = kwargs.get(
+            'scality_mount_point_base', "").rstrip('/')
+        if not self._mount_base:
+            raise exception.InvalidParameterValue(
+                err=_('scality_mount_point_base required'))
+        self._mount_options = None
+
+    def get_mount_point(self, device_name):
+        return os.path.join(self._mount_base,
+                            device_name,
+                            "00")
+
+    def mount(self, share, flags=None):
+        """Mount the Scality ScaleOut FS.
+
+        The `share` argument is ignored because you can't mount several
+        SOFS at the same type on a single server. But we want to keep the
+        same method signature for class inheritance purpose.
+        """
+        if self._mount_base in self._read_mounts():
+            LOG.info(_LI('Already mounted: %s'), self._mount_base)
+            return
+        self._execute('mkdir', '-p', self._mount_base, check_exit_code=0)
+        super(ScalityRemoteFsClient, self)._do_mount(
+            'sofs', '/etc/sfused.conf', self._mount_base)
